#include "define.h"
#include "configure.h"
#include <fstream>
#include "strutil.h"
#include "StringTokenizer.h"
#include <algorithm>
#include "dialog.h"

using namespace strutil;
using namespace std;

namespace MLS {

Configure &config = Configure::getInstance();
std::string EmptyString;

bool Configure::Load( const char * file)
{
	ifstream in(file);	
	if (!in) return false;
	
    LOG("Load File Name :: [%s]", file);
    
	filename = file;
	
	string line, var, val;
	
	while(!getline(in, line).eof())
	{
		if (line.empty()) continue;
		if (line[0] == '#') continue;
		
		string::size_type p = line.find('=');
		if (p == string::npos) continue;
		
		// . var와 값을 찾는다.
		var = tolower(chop(line.substr(0, p)));
		val = chop(line.substr(p+1));
		
		//cout << var << ' ' << val << endl;
		
		if (var.substr(0, 4) == "ext[")
		{
			StringTokenizer st(getbetween(var, '[', ']'), "; ");
			while (st.Next()) 
			{				
				ext_bind[tolower(st.Get())] = val;
			}
		}
		else env_map[var] = Entry(val, false);
	}			
	
	return true;
}

bool Configure::Save()
{// 바로 그 파일에다 저장한다..	
	string bakfile = filename + ".bak";
	
	unlink(bakfile.c_str());
	if (rename(filename.c_str(), bakfile.c_str()) == -1 ) return false;
		
	ifstream in(bakfile.c_str());
	ofstream out(filename.c_str());
	
	if (!in) return false;
	if (!out) return false;
	
	// Modified 된 것을 정리
	MapType::iterator i = env_map.begin(), end = env_map.end();	
	MapType mod_list, dup_list;
	
	for(;i!=end;++i)
		if (i->second.modified) mod_list.insert(*i);
	
	string line, var, val;
	
	while( !getline(in, line).eof() ) 
	{
		if (line.empty() || line[0] == '#' || line[0] == '$') 
		{
			out << line << endl;						
			continue;
		}
		
		string::size_type p = line.find('=');		
		if (p == string::npos)
		{
			out << line << endl;
			continue;
		}
		
		var = chop(line.substr(0, p));
		val = chop(line.substr(p+1));
		
		i = mod_list.find(tolower(var));
		
		if ( i != mod_list.end() )
		{						
			out << var << " = " << (*i).second.val << endl;
			dup_list.insert(*i);
			mod_list.erase(i);
		}
		else if ((i = dup_list.find(tolower(var))) != dup_list.end())
		{
			out << '#' << line << endl;
	 	}
		else out << line << endl;
		
	}
	// 남은 것들 처리
	for (i = mod_list.begin(); i!=mod_list.end(); i++)
		out << i->first << " = " << i->second.val << endl;
	
	in.close();
	out.close();

	return true;
}

const std::string &Configure::GetValue(const std::string& var)
{
	MapType::iterator i = env_map.find(strutil::tolower(var));
	if (i == env_map.end())
	{
		MsgBox(2, "Emtpy!!", "%s", var.c_str());
		return EmptyString;
	}
	
	return (*i).second.val;		
}

bool Configure::SetValue(const std::string &var, const std::string &val, bool bSave)
{
	std::string v = strutil::tolower(var);
	MapType::iterator i = env_map.find(v);
	if (i == env_map.end())
		env_map.insert(MapType::value_type(v, Entry (val, bSave)));
	else
	{
		(*i).second.modified = (bSave && (*i).second.val != val);
		(*i).second.val = val;			
	}
	
}

bool Configure::GetBool( const std::string &var )
{
	MapType::iterator i = env_map.find(strutil::tolower(var));		
	if (i == env_map.end()) return false;
	
	return (strutil::tolower((*i).second.val) == "on");
}

bool Configure::SetBool(const std::string &var, bool value, bool bSave)
{
	int val;
	if (value) val = 1; else val = 0;
		
	const char *str[2] = {"Off", "On"};
	std::string v = strutil::tolower(var);
	
	MapType::iterator i = env_map.find(v);
	
	if (i == env_map.end())
		env_map.insert(MapType::value_type(v, Entry (str[val], bSave)));
	else
	{
		(*i).second.modified = (bSave && (*i).second.val != str[val]);
		(*i).second.val = str[val];			
	}
	
}



IConfigurable::IConfigurable()
{
	this->UpdateConfig();
	
}

IConfigurable::~IConfigurable()
{
	this->SaveConfig();	
}

void IConfigurable::SaveConfig()
{
}

void IConfigurable::UpdateConfig()
{
}


};



