#ifndef MLSCONFIGURE_H
#define MLSCONFIGURE_H

#include <string>
#include <map>
#include <vector>
#include "drawutil.h"
#include "strutil.h"

namespace MLS {

extern std::string EmptyString;

class Configure
{
	struct Entry
	{
		bool modified;
		std::string val;
		Entry() : modified(false) { }
		Entry(const std::string &_val, bool b) : modified(b), val(_val) {}
	};
	
	typedef std::map<std::string, Entry> MapType;
	
	std::string filename;
	
	MapType env_map;
		
	// singleton
	Configure() { }
	~Configure(){ }
	
public:
	
	std::map<std::string, std::string> ext_bind;
	
	static Configure &getInstance()
	{
		static Configure conf;
		return conf;
	}
	
	bool Load(const char *);
	bool Save();
	
	const std::string &GetValue(const std::string& var);
	bool SetValue(const std::string &var, const std::string &val, bool bSave = true);
	
	bool GetBool(const std::string &);
	bool SetBool(const std::string &var, bool val, bool bSave = true);
};

class IConfigurable
{
public:
	IConfigurable();
	virtual ~IConfigurable();
	virtual void UpdateConfig();
	virtual void SaveConfig();
};

extern Configure &config;

};

#endif
